#!/bin/bash

ADJUST_REVISION=1

while test "$1" != "" ; do
    case "$1" in
	version)
	    ADJUST_REVISION=0
	    ;;
	*)
	    echo "Usage: $0 [version]"
	    echo " version  - don't adjust revision before building release"
	    exit 1
	    ;;
    esac
    shift
done

cvs commit -m "changes for building release" .

if [ $ADJUST_REVISION -eq 1 ]; then
    make up_revision
fi

VERSION=`awk '/^#define DIAMOND_GIRL_VERSION /{print $3}' < src/version.h`
REVISION=`awk '/^#define DIAMOND_GIRL_REVISION /{print $3}' < src/version.h`

echo "Building release for Diamond Girl v$VERSION.$REVISION"

echo "" >> Changelog
echo "Release $VERSION.$REVISION on `date --utc`" >> Changelog
if [ -f "new_changes" ]; then cat new_changes >> Changelog ; fi
echo "" > new_changes
touch Changelog
cvs commit -m "Changelog update." Changelog new_changes

cvs tag -c -R release-$VERSION-$REVISION || exit 1


TMPDIR=diamond_girl-$VERSION.$REVISION
cvs export -R -r release-$VERSION-$REVISION -d /tmp/$TMPDIR diamond_girl &&
    tar -cj -f html/diamond_girl-$VERSION.$REVISION.tar.bz -C /tmp $TMPDIR
#( cd /tmp/$TMPDIR &&
#    make diamond_girl &&
#    strip diamond_girl &&
#    mv diamond_girl diamond_girl-linux-i386 &&
#    make clean
#) && tar -cj -f html/diamond_girl-linux-i386.tar.bz -C /tmp $TMPDIR
rm -rf /tmp/$TMPDIR

ls -l html/diamond_girl-$VERSION.$REVISION.tar.bz
# html/diamond_girl-linux-i386.tar.bz


VERSION=$VERSION REVISION=$REVISION sh ./build_html.sh

VERSION=$VERSION REVISION=$REVISION sh ./build_rpm.sh

echo "Done."
