/*
  Diamond Girl - Game where player collects diamonds.
  Copyright (C) 2005  Joni Yrjana
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


  Complete license can be found in the LICENSE file.
*/

#ifndef UI_H_
#define UI_H_

#include <stdlib.h>

enum UI_BUTTON { UI_BUTTON_LEFT, UI_BUTTON_MIDDLE, UI_BUTTON_RIGHT };
enum UI_OBJECT_STATE { UI_OBJECT_STATE_NORMAL, UI_OBJECT_STATE_SELECTED, UI_OBJECT_STATE_ACTIVATED };

struct ui_object;
typedef void (*ui_func_draw_t)(struct ui_object * this);
typedef void (*ui_func_activate_t)(struct ui_object * this, enum UI_BUTTON button);
typedef void (*ui_func_activate_at_t)(struct ui_object * this, enum UI_BUTTON button, unsigned int x, unsigned int y);
typedef void (*ui_func_mouse_at_t)(struct ui_object * this, unsigned int x, unsigned int y);

struct ui_object
{

  unsigned char dirty_;
  enum UI_OBJECT_STATE state_;

  char ** user_data_names_;
  void ** user_data_;
  size_t  user_data_size_;

  unsigned int x_, y_;
  unsigned int width_, height_;
  ui_func_draw_t        draw_function_;
  ui_func_activate_t    activate_function_;
  ui_func_activate_at_t activate_at_function_;
  ui_func_activate_t    inactivate_function_;
  ui_func_mouse_at_t    mouse_at_function_;
};


extern int ui_initialize(void);
extern void ui_cleanup(void);

extern void ui_draw(void);
extern void ui_process_mouse_move(unsigned int mouse_x, unsigned int mouse_y);
extern void ui_process_mouse_button(unsigned int mouse_x, unsigned int mouse_y, enum UI_BUTTON button, unsigned char release);

extern struct ui_object * ui_object(unsigned int x, unsigned int y, unsigned int width, unsigned int height);
extern struct ui_object * ui_object_delete(struct ui_object * object);

/* getters */
extern struct ui_object *    ui_object_selected(void);
extern enum UI_OBJECT_STATE  ui_object_state(struct ui_object * object);
extern unsigned int          ui_object_x(struct ui_object * object);
extern unsigned int          ui_object_y(struct ui_object * object);
extern unsigned int          ui_object_width(struct ui_object * object);
extern unsigned int          ui_object_height(struct ui_object * object);
extern void *                ui_object_user_data_pointer(struct ui_object * object, const char * name);
extern long                  ui_object_user_data_long(struct ui_object * object, const char * name);
extern unsigned long         ui_object_user_data_ulong(struct ui_object * object, const char * name);
extern ui_func_draw_t        ui_object_draw_function(struct ui_object * object);
extern ui_func_activate_t    ui_object_activate_function(struct ui_object * object);
extern ui_func_activate_at_t ui_object_activate_at_function(struct ui_object * object);
extern ui_func_activate_t    ui_object_inactivate_function(struct ui_object * object);
extern ui_func_mouse_at_t    ui_object_mouse_at_function(struct ui_object * object);
/* setters */
extern void                  ui_object_set_selected(struct ui_object * object);
extern void                  ui_object_set_dirty(struct ui_object * object);
extern unsigned int          ui_object_set_x(struct ui_object * object, unsigned int x);
extern unsigned int          ui_object_set_y(struct ui_object * object, unsigned int y);
extern unsigned int          ui_object_set_width(struct ui_object * object, unsigned int width);
extern unsigned int          ui_object_set_height(struct ui_object * object, unsigned int height);
extern void *                ui_object_set_user_data_pointer(struct ui_object * object, const char * name, void * user_data);
extern long                  ui_object_set_user_data_long(struct ui_object * object, const char * name, long user_data);
extern unsigned long         ui_object_set_user_data_ulong(struct ui_object * object, const char * name, unsigned long user_data);
extern ui_func_draw_t        ui_object_set_draw_function(struct ui_object * object, ui_func_draw_t draw_function);
extern ui_func_activate_t    ui_object_set_activate_function(struct ui_object * object, ui_func_activate_t activate_function);
extern ui_func_activate_at_t ui_object_set_activate_at_function(struct ui_object * object, ui_func_activate_at_t activate_at_function);
extern ui_func_activate_t    ui_object_set_inactivate_function(struct ui_object * object, ui_func_activate_t inactivate_function);
extern ui_func_mouse_at_t    ui_object_set_mouse_at_function(struct ui_object * object, ui_func_mouse_at_t mouse_at_function);


#endif

